# Модули
Модуль - функционально законченный фрагмент программы. Модуль состоит только из компонент и имеет зависимостей на другие модули.

Желательно чтобы на каждый под проект создавался как минимум один модуль. Не стоит объявлять один модуль на несколько подпроектов.

## Объявление
Для объявления модуля нужно реализовать протокол `DIModule`:
```Swift
class YourModule: DIModule {
  var components: [DIComponent] = [...]
  
  var dependencies: [DIModule] = [...]
}
```

У модуля есть два поля:
* components - список всех компонентов, из которых состоит модуль.
* dependencies - список зависимостей на другие модули.

### Область видимости компонент

В прошлой главе, был описан синтаксис, позволяющий изменить область видимости. Чтобы лучше понимать, что это такое, достаточно запомнить два правила:
* Регистрации, проходящие в `public` компоненте - включается в сам модуль и во все модули, которые зависят от этого модуля.
* Регистрации, проходящие в `internal` компоненте - включается только в сам модуль.

## Регистрация
После того как мы объявили модуль, его нужно зарегистрировать для дальнейшего использования:
```Swift
builder.register(module: YourModule())
```
При регистрации модуля, регистрируются все указанные компоненты и все указанные зависимости.

!! Типы со временем жизни: `single`,`lazySingle`,`weakSingle` единственные для всей программы, а не модуля.


#### [Главная](main.md)
#### [Предыдущая глава "Компоненты"](component.md)
#### [Следующая глава "Позднее связывание"](lateBinding.md)
